@@ -1,10 +1,14 @@
 import pandas as pd
 from os import path
+import os
 import json
 from pyensembl import Genome
 import requests
 from gwaslab.Log import Log
-
+import tarfile
+from gwaslab.download import download_ref
+from gwaslab.download import check_and_download
+from gwaslab.download import get_path
 #hard-coded data
 def get_chr_to_NC(build,inverse=False):
     #https://www.ncbi.nlm.nih.gov/assembly/GCF_000001405.13
@@ -207,57 +211,43 @@ def get_recombination_rate(chrom, build="19"):
     #http://ftp.1000genomes.ebi.ac.uk/vol1/ftp/technical/working/20110106_recombination_hotspots/
     if build=="19":
         data_path =  path.dirname(__file__) + '/data/recombination/hg19/genetic_map_GRCh37_chr'+str(chrom)+'.txt.gz'
-        recombination_rate = pd.read_csv(data_path,sep="\t")
+        if path.exists(data_path):
+            recombination_rate = pd.read_csv(data_path,sep="\t")
+        else:
+            if not path.exists(path.dirname(__file__) + '/data/recombination/hg19/'):
+                os.makedirs(path.dirname(__file__) + '/data/recombination/hg19/')
+            download_ref("recombination_hg19",directory = path.dirname(__file__)+'/data/recombination/hg19/')
+            file = tarfile.open(path.dirname(__file__)+'/data/recombination/hg19/recombination_hg19.tar.gz')
+            file.extractall(path.dirname(__file__)+'/data/recombination/hg19/')
+            file.close()
+            recombination_rate = pd.read_csv(data_path,sep="\t")
     return recombination_rate
 
 
 ####################################################################################################################
 def get_gtf(chrom, build="19",source="ensembl"):
     if source=="ensembl":
-        if build=="19":
-            data_path =  path.dirname(__file__) + '/data/Ensembl/release75/Homo_sapiens.GRCh37.75.gtf.gz'
+        if build=="19": 
+            data_path = check_and_download("ensembl_hg19_gtf")
             gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#",dtype={0:"string"})
             gtf = gtf.loc[gtf[0]==chrom,:]
         if build=="38":
-            data_path =  path.dirname(__file__) + '/data/Ensembl/release107/Homo_sapiens.GRCh38.107.chr.gtf.gz'
+            data_path = check_and_download("ensembl_hg38_gtf")
             gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#",dtype={0:"string"})
             gtf = gtf.loc[gtf[0]==chrom,:]
     if source=="refseq":
         chrom_NC = get_chr_to_NC(build=build)[str(chrom)]
         if build=="19":
-            data_path =  path.dirname(__file__) + '/data/RefSeq/GRCh37/GRCh37_latest_genomic.gtf.gz'
+            data_path = check_and_download("refseq_hg19_gtf")
             gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#")
             gtf = gtf.loc[gtf[0]==chrom_NC,:]
         if build=="38":
-            data_path =  path.dirname(__file__) + '/data/RefSeq/GRCh38/GRCh38_latest_genomic.gtf.gz'
+            data_path = check_and_download("refseq_hg38_gtf")
             gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#")
             gtf = gtf.loc[gtf[0]==chrom_NC,:]
         gtf[0] = str(chrom)
     return gtf
 
-#def get_gtf_chr(chrom, build="19"):
-#    if build=="19":
-#        data_path =  path.dirname(__file__) + '/data/Ensembl/release75/Homo_sapiens.GRCh37.75.chr'+str(chrom)+'.gtf.gz'
-#        gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#")
-#    if build=="38":
-#        data_path =  path.dirname(__file__) + '/data/Ensembl/release107/Homo_sapiens.GRCh38.107.chr'+str(chrom)+'.gtf.gz'
-#        gtf = pd.read_csv(data_path,sep="\t",header=None,comment="#")
-#    return gtf
-
-def gtf_index():
-    if build=="19":
-        data_path =  path.dirname(__file__) + '/data/Ensembl/release75/Homo_sapiens.GRCh37.75.gtf.gz'
-        data = Genome(  reference_name='GRCh37',
-                        annotation_name='hg19_genes',
-                        gtf_path_or_url=data_path)
-        data.index()
-    if build=="38":
-        data_path =  path.dirname(__file__) + '/data/Ensembl/release107/Homo_sapiens.GRCh38.107.chr.gtf.gz'
-        data = Genome(  reference_name='GRCh38',
-                        annotation_name='hg38_genes',
-                        gtf_path_or_url=data_path)
-        data.index()
-
 
 ####################################################################################################################
 
@@ -269,7 +259,7 @@ def update_formatbook(log=Log()):
     data_path =  path.dirname(__file__) + '/data/formatbook.json'
     with open(data_path, 'wb') as file:
         file.write(r.content)
-    book=json.load(open("formatbook.json"))
+    book=json.load(open(data_path))
     available_formats = list(book.keys())
     available_formats.sort()
     log.write("Available formats:",",".join(available_formats))
@@ -278,14 +268,14 @@ def update_formatbook(log=Log()):
          
 def list_formats(log=Log()):
     data_path =  path.dirname(__file__) + '/data/formatbook.json'
-    book=json.load(open("formatbook.json"))
+    book=json.load(open(data_path))
     available_formats = list(book.keys())
     available_formats.sort()
     log.write("Available formats:",",".join(available_formats))    
 
 def check_format(fmt,log=Log()):
     data_path =  path.dirname(__file__) + '/data/formatbook.json'
-    book=json.load(open("formatbook.json"))
+    book=json.load(open(data_path))
     log.write("Available formats:",end="")
     for i in book[fmt].keys():
         log.write(i,end="")
@@ -293,12 +283,12 @@ def check_format(fmt,log=Log()):
     for i in book[fmt].values():
         log.write(i,end="")
         
-####################################################################################################################       
+####################################################################################################################   
         
 def gwaslab_info():
     dic={
-   "version":"3.3.3",
-   "release_date":"20221029"
+   "version":"3.3.5",
+   "release_date":"20221104"
     }
     return dic       
         
