@@ -22,6 +22,7 @@ from gwaslab.retrievedata import rsidtochrpos
 from gwaslab.retrievedata import parallelizeassignrsid
 from gwaslab.retrievedata import parallelinferstrand
 from gwaslab.retrievedata import parallelrsidtochrpos
+from gwaslab.filtervalue import filtervalues
 from gwaslab.filtervalue import filterout
 from gwaslab.filtervalue import filterin
 from gwaslab.filtervalue import filterregionin
@@ -75,13 +76,16 @@ class Sumstats():
              other=[],
              direction=None,
              verbose=True,
+             study=None,
              build="99",
              **readargs):
-        
-        self.build = build
-        self.meta = {"Genome build":build,
-                 "Name":"Sumstats_1","StudyType":"Unknown"}
+
         self.data = pd.DataFrame()
+        self.build = build
+        self.meta = {"GenomeBuild":build,
+                     "Name":"Sumstats_1",
+                     "StudyType":"Unknown"}
+        self.meta["Study"]=study
         self.log = Log()
         
         #preformat the data
@@ -110,47 +114,17 @@ class Sumstats():
           OR_95L=OR_95L,
           OR_95U=OR_95U,
           direction=direction,
+          study=study,
           build=build,
           status=status,
           other=other,
           verbose=verbose,
           readargs=readargs,
           log=self.log)
-        gc.collect()     
+        gc.collect()   
+
 #### healper #################################################################################
-    #value to arguement name 
-    def get_columns(self, columns=[]):
-        dic = {
-            "SNPID": "snpid",
-            "rsID": "rsid",
-            "CHR": "chrom",
-            "POS": "pos",
-            "NEA": "nea",
-            "EA": "ea",
-            "EAF": "eaf",
-            "N": "n",
-            "BETA": "beta",
-            "SE": "se",
-            "Z": "z",
-            "CHISQ": "chisq",
-            "P": "p",
-            "MLOG10P": "mlog10p",
-            "INFO": "info",
-            "OR": "OR",
-            "OR_95L": "OR_95L",
-            "OR_95U": "OR_95U",
-        }
-        other=[]
-        col_subset={}
-        for key in columns:
-            if key not in dic.keys():
-                other.append(key)
-            else:
-                col_subset[dic[key]] = key
-        if len(other)>0: col_subset["other"] = other      
-        return col_subset
 
-##########################################################################################
     def update_meta(self):
         self.meta["Number_of_variants"]=len(self.data)
         if "CHR" in self.data.columns:
@@ -159,12 +133,14 @@ class Sumstats():
             self.meta["Min_P"]=np.min(self.data["P"])
     def summary(self):
         return summarize(self.data)
+
     def lookup_status(self,status="STATUS"):
         return lookupstatus(self.data[status])
         
 # QC ######################################################################################
     #clean the sumstats with one line
-    def basic_check(self,remove=False,
+    def basic_check(self,
+                    remove=False,
                     n_cores=1,
                     fixid_args={},
                     removedup_args={},
@@ -330,46 +306,56 @@ class Sumstats():
         self.data = parallelrsidtochrpos(self.data,log=self.log,**args)
     def liftover(self,**args):
         self.data = parallelizeliftovervariant(self.data,log=self.log,**args)
+    ############################################################################################################
+    
     def sort_coordinate(self,**sort_args):
         self.data = sortcoordinate(self.data,log=self.log,**sort_args)
     def sort_column(self,**args):
         self.data = sortcolumn(self.data,log=self.log,**args)
+    
     ############################################################################################################
     def fill_data(self, **args):
         self.data = filldata(self.data,**args)
+    
     def infer_build(self,**args):
-        self.data = inferbuild(self.data,**args)
+        self.data, self.meta["GenomeBuild"] = inferbuild(self.data,**args)
 
 # utilities ############################################################################################################
-    def filter_out(self, inplace=True, **args):
+    def filter_value(self, expr, inplace=False, **args):
+        if inplace is False:
+            return filtervalues(self.data, expr,**args)
+        else:
+            self.data = filtervalues(self.data, expr,log=self.log,**args)
+    
+    def filter_out(self, inplace=False, **args):
         if inplace is False:
             return filterout(self.data,**args)
         else:
             self.data = filterout(self.data,log=self.log,**args)
             
-    def filter_in(self, inplace=True, **args):
+    def filter_in(self, inplace=False, **args):
         if inplace is False:
             return filterin(self.data,**args)
         else:
             self.data = filterin(self.data,log=self.log,**args)
-    def filter_region_in(self, inplace=True, **args):
+    def filter_region_in(self, inplace=False, **args):
         if inplace is False:
             return filterregionin(self.data,**args)
         else:
             self.data = filterregionin(self.data,log=self.log,**args)
-    def filter_region_out(self, inplace=True, **args):
+    def filter_region_out(self, inplace=False, **args):
         if inplace is False:
             return filterregionout(self.data,**args)
         else:
             self.data = filterregionout(self.data,log=self.log,**args)
     
-    def random_variants(self,in_place=False,n=1,**args):
-        if in_place is True:
+    def random_variants(self,inplace=False,n=1,**args):
+        if inplace is True:
             self.data = sampling(self.data,n=n,log=self.log,**args)
         else:
             return sampling(self.data,n=n,log=self.log,**args)
-    
-###########################################################################################################################            
+
+
     def check_af(self,**args):
         self.data = parallelecheckaf(self.data,log=self.log,**args)
       
@@ -453,29 +439,6 @@ class Sumstats():
         #return scalar
         self.meta["Genomic inflation factor"] = output
         return output    
-    
-    def exclude_hla(self, **args):
-        is_hla = (self.data["CHR"] == "6") & (self.data["POS"] > 25000000) & (
-            self.data["POS"] < 34000000)
-        output = self.data.loc[~is_hla]  #not in hla
-        # return sumstats object    
-        return Sumstats(output, **self.get_columns(), verbose=False)
-
-    def extract_hla(self, **args):
-        is_hla = (self.data["CHR"] == "6") & (self.data["POS"] > 25000000) & (
-            self.data["POS"] < 34000000)
-        output = self.data.loc[is_hla]  #in hla
-        # return sumstats object    
-        return Sumstats(output, **self.get_columns(self.data.columns), verbose=False)
-    
-    def extract_chr(self,chrom="1", **args):
-        is_chr = (self.data["CHR"] == str(chrom))
-        output = self.data.loc[is_chr]  #in hla
-        # return sumstats object    
-        return Sumstats(output, **self.get_columns(self.data.columns), verbose=False)
-    
-    
-
         
 # to_format ###############################################################################################       
     def to_format(self,
@@ -495,6 +458,7 @@ class Sumstats():
               xymt_number=False,
               xymt=["X","Y","MT"],
               chr_prefix=None,
+              md5sum=False,
               bgzip=False,
               tabix=False):
         
@@ -589,8 +553,9 @@ class Sumstats():
                   meta = self.meta,
                   bgzip=bgzip,
                   tabix=tabix,
+                  md5sum=md5sum,
                   xymt_number=xymt_number,
                   xymt=xymt)
         if output_log is True:
             if verbose: self.log.write(" -Saving log file:",path + "."+ suffix +".log")
-        if verbose: self.log.write("Finished outputting successfully!")
+        if verbose: self.log.write("Finished outputting successfully!")
\ No newline at end of file
