@@ -1,4 +1,5 @@
 import re
+import gc
 #import gwaslab as gl
 import pandas as pd
 import numpy as np
@@ -28,7 +29,7 @@ from gwaslab.CommonData import get_chr_list
 #sortcolumn
 
 ###############################################################################################################
-#20220514 
+# 20220514 
 
 def fixID(sumstats,
        snpid="SNPID",rsid="rsID",chrom="CHR",pos="POS",nea="NEA",ea="EA",status="STATUS",
@@ -263,11 +264,11 @@ def fixID(sumstats,
     if verbose: log.write("Finished checking IDs successfully!")
     return sumstats
 
-###############################################################################################################
+""
+
 
-    
 ###############################################################################################################
-#20220514 
+# 20220514 
 def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea="NEA",rsid="rsID",keep='first',keep_col="P",remove=False,keep_ascend=True,verbose=True,log=Log()):
     '''
     remove duplicate SNPs based on 1. SNPID,
@@ -334,7 +335,7 @@ def removedup(sumstats,mode="dm",chrom="CHR",pos="POS",snpid="SNPID",ea="EA",nea
     return sumstats
 
 ###############################################################################################################
-#20220514
+# 20220514
 def fixchr(sumstats,chrom="CHR",status="STATUS",add_prefix="",x="X",y="Y",mt="MT",remove=False, verbose=True,log=Log()):
         if check_col(sumstats,chrom,status) is not True:
             if verbose: log.write(".fix_chr: Specified not detected..skipping...")
@@ -411,9 +412,9 @@ def fixchr(sumstats,chrom="CHR",status="STATUS",add_prefix="",x="X",y="Y",mt="MT
         if verbose: log.write("Finished fixing chromosome notation successfully!")
         return sumstats
     
-    
+
 ###############################################################################################################    
-#20220514
+# 20220514
 def fixpos(sumstats,pos="POS",status="STATUS",remove=False, verbose=True,limit=250000000, log=Log()):
         if check_col(sumstats,pos,status) is not True:
             if verbose: log.write(".fix_pos: Specified not detected..skipping...")
@@ -450,9 +451,9 @@ def fixpos(sumstats,pos="POS",status="STATUS",remove=False, verbose=True,limit=2
         if verbose: log.write(" -Converted all position to datatype Int64.")
         if verbose: log.write("Finished fixing basepair position successfully!")
         return sumstats
-    
+
 ###############################################################################################################    
-#20220514
+# 20220514
 def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=True,log=Log()):
         # remove variants with alleles other than actgACTG
         if check_col(sumstats,ea,nea,status) is not True:
@@ -462,23 +463,31 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
         if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
         
         #if (ea not in sumstats.columns) or (nea not in sumstats.columns):
-        sumstats.loc[:,ea]=pd.Categorical(sumstats[ea].str.upper(),categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])) 
-        sumstats.loc[:,nea]=pd.Categorical(sumstats[nea].str.upper(),categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])) 
-        
+        categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])|set("N")
+        sumstats.loc[:,ea]=pd.Categorical(sumstats[ea].str.upper(),categories = categories) 
+        sumstats.loc[:,nea]=pd.Categorical(sumstats[nea].str.upper(),categories = categories) 
         all_var_num = len(sumstats)
-        bad_ea = sumstats[ea].str.contains("[^actgACTG]")
-        bad_nea = sumstats[nea].str.contains("[^actgACTG]")
+        bad_ea = sumstats[ea].str.contains("[^actgACTG]",na=True)
+        bad_nea = sumstats[nea].str.contains("[^actgACTG]",na=True)
         good_ea  = ~bad_ea
         good_nea = ~bad_nea
         is_eanea_na = sumstats[ea].isna() |  sumstats[nea].isna()
         is_invalid = bad_ea | bad_nea | (sumstats[nea] == sumstats[ea])
-        exclude  = sum(bad_nea | bad_ea)
+        exclude  = bad_nea | bad_ea
+        if verbose: 
+            if len(set(sumstats.loc[bad_ea,ea].head())) >0:
+                log.write(" -A look at the non-ATCG EA:",set(sumstats.loc[bad_ea,ea].head()),"...") 
+            if len(set(sumstats.loc[bad_nea,nea].head())) >0:
+                log.write(" -A look at the non-ATCG NEA:",set(sumstats.loc[bad_nea,nea].head()),"...") 
         if remove is True:
-            sumstats = sumstats.loc[good_ea & good_nea,:].copy()
-            if verbose: log.write(" -Removed "+str(exclude)+" variants with alleles that contain bases other than A/C/T/G .")  
-        
-        sumstats.loc[:,ea]=pd.Categorical(sumstats[ea].str.upper(),categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])) 
-        sumstats.loc[:,nea]=pd.Categorical(sumstats[nea].str.upper(),categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])) 
+            sumstats = sumstats.loc[(good_ea & good_nea),:].copy()
+            if verbose: log.write(" -Removed "+str(sum(exclude))+" variants with alleles that contain bases other than A/C/T/G .")  
+        else:
+            sumstats.loc[:,[ea,nea]] = sumstats.loc[:,[ea,nea]].fillna("N")
+            if verbose: log.write(" -Detected "+str(sum(exclude))+" variants with alleles that contain bases other than A/C/T/G .") 
+        categories = set(sumstats.loc[:,ea])|set(sumstats.loc[:,nea])|set("N")
+        sumstats.loc[:,ea]=pd.Categorical(sumstats[ea].str.upper(),categories = categories) 
+        sumstats.loc[:,nea]=pd.Categorical(sumstats[nea].str.upper(),categories = categories) 
         if verbose: log.write(" -Converted all bases to string datatype and UPPERCASE.")
         
         is_eanea_fixed = good_ea | good_nea
@@ -494,12 +503,12 @@ def fixallele(sumstats,ea="EA", nea="NEA",status="STATUS",remove=False,verbose=T
         sumstats.loc[is_eanea_fixed&is_indel,status]         = vchange_status(sumstats.loc[is_eanea_fixed&is_indel, status],      5,"9","4")
         sumstats.loc[is_eanea_fixed&is_normalized,status]      = vchange_status(sumstats.loc[is_eanea_fixed&is_normalized, status],  5,"4","3")
         remain_var_num = len(sumstats)
-              
+        gc.collect()
         if verbose: log.write("Finished fixing allele successfully!")
         return sumstats
 
 ###############################################################################################################   
-#20220721
+# 20220721
 
 def parallelnormalizeallele(sumstats,pos="POS",nea="NEA",ea="EA" ,status="STATUS",n_cores=1,verbose=True,log=Log()):
     if check_col(sumstats,pos,ea,nea,status) is not True:
@@ -601,10 +610,11 @@ def normalizevariant(pos,a,b,status):
     if len(a)==1 or len(b)==1:
         return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"0"+status_end
     return pos+pos_change,a[pointer_a_l:pointer_a_r+1],b[pointer_b_l:pointer_b_r+1],status_pre+"3"+status_end
-###############################################################################################################
+""
+
 
 ###############################################################################################################
-#20220426
+# 20220426
 def sanitycheckstats(sumstats,
                      coltocheck=["P","MLOG10P","Z","BETA","SE","EAF","CHISQ","N","OR","OR_95L","OR_95U","STATUS"],
                      n=(0,float("Inf")),
@@ -769,7 +779,7 @@ def sanitycheckstats(sumstats,
     return sumstats
 
 ###############################################################################################################
-#20220426
+# 20220426
 def get_reverse_complementary_allele(a):
     dic = str.maketrans({
        "A":"T",
@@ -904,10 +914,11 @@ def flipallelestats(sumstats,status="STATUS",verbose=True,log=Log()):
         sumstats.loc[matched_index,status] = vchange_status(sumstats.loc[matched_index,status], 7, "5","2")
     if verbose: log.write("Finished converting successfully!")
     return sumstats
-###############################################################################################################
+""
+
 
 ###############################################################################################################
-#20220426
+# 20220426
 def liftover_snv(row,chrom,converter,to_build):
     status_pre=""
     status_end=row[1][2]+"9"+row[1][4]+"99"  
@@ -986,7 +997,7 @@ def parallelizeliftovervariant(sumstats,n_cores=1,chrom="CHR", pos="POS", from_b
     return sumstats
 
 ###############################################################################################################
-#20220426
+# 20220426
 def sortcoordinate(sumstats,chrom="CHR",pos="POS",reindex=True,verbose=True,log=Log()):
     if check_col(sumstats,chrom,pos) is not True:
         if verbose: log.write(".liftover(): specified columns not detected..skipping...")
@@ -1034,10 +1045,10 @@ def sortcoordinate(sumstats,chrom="CHR",pos="POS",reindex=True,verbose=True,log=
     if verbose: log.write("Finished sorting genome coordinates successfully!")
     return sumstats
 ###############################################################################################################
-#20220426
+# 20221105
 def sortcolumn(sumstats,verbose=True,log=Log(),order = [
         "SNPID","rsID", "CHR", "POS", "EA", "NEA", "EAF", "BETA", "SE", "Z",
-        "CHISQ", "P", "MLOG10P", "OR", "OR_SE", "OR_95L", "OR_95U", "INFO", "N","DIRECTION","STATUS"
+        "CHISQ", "P", "MLOG10P", "OR", "OR_95L", "OR_95U", "INFO", "N","DIRECTION","STATUS"
            ]):
     if verbose: log.write("Start to reorder the columns...")
     if verbose: log.write(" -Current Dataframe shape :",len(sumstats)," x ", len(sumstats.columns))   
@@ -1052,9 +1063,6 @@ def sortcolumn(sumstats,verbose=True,log=Log(),order = [
     if verbose: log.write("Finished sorting columns successfully!")
     return sumstats
 
-################################################################################################################
-
-################################################################################################################
 
 def check_col(df,*args):
     not_in_df=[]
