@@ -5,16 +5,18 @@ from gwaslab.Log import Log
 from gwaslab.CommonData import get_chr_to_number
 from gwaslab.CommonData import get_number_to_chr
 from gwaslab.CommonData import get_chr_to_NC
-from gwaslab.CommonData import gtf_index
+from gwaslab.download import check_and_download
+from gwaslab.gwascatalog import gwascatalog_trait
 from pyensembl import EnsemblRelease
 from pyensembl import Genome
 from os import path
 from gwaslab.fill import fill_p
+import gc
 
-#getsig
-#closest_gene
-#annogene
-#getnovel
+# getsig
+# closest_gene
+# annogene
+# getnovel
 
 def getsig(insumstats,
            id,
@@ -124,6 +126,7 @@ def getsig(insumstats,
                source=source,
                verbose=verbose)
     if verbose: log.write("Finished extracting lead variants successfully!")
+    gc.collect()
     return output.copy()
 
 
@@ -185,8 +188,8 @@ def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50,source="ense
             return distance,"intergenic"
         else:
             return 0,",".join(gene)
-        
-        
+
+
 def annogene(
            insumstats,
            id,
@@ -204,12 +207,14 @@ def annogene(
     if source == "ensembl":
         if build=="19":
             #data = EnsemblRelease(75)
-            if verbose:log.write(" -Assigning Gene name using built-in Ensembl Release",75 , " (hg19)")
+            if verbose:log.write(" -Assigning Gene name using Ensembl Release hg19")
             #zcat Homo_sapiens.GRCh37.75.gtf.gz| 
             #grep -E 'processed_transcript|protein_coding|_gene' 
             #| gzip >Homo_sapiens.GRCh37.75.processed.chr.gtf.gz     
-            gtf_path = path.dirname(__file__) + '/data/Ensembl/release75/Homo_sapiens.GRCh37.75.protein_coding.gtf.gz'
-            gtf_db_path = path.dirname(__file__) + '/data/Ensembl/release75/Homo_sapiens.GRCh37.75.protein_coding.gtf.db'
+            
+            gtf_path = check_and_download("ensembl_hg19_gtf_protein_coding")
+            gtf_db_path = gtf_path[:-2]+"db"
+            
             data = Genome(
                 reference_name='GRCh37',
                 annotation_name='hg19_gene',
@@ -220,9 +225,9 @@ def annogene(
                 list(output.apply(lambda x:closest_gene(x,data=data,source=source), axis=1)), 
                 index=output.index).values
         elif build=="38":
-            if verbose:log.write(" -Assigning Gene name using built-in Ensembl Release",107 , " (hg38)")
-            gtf_path = path.dirname(__file__) + '/data/Ensembl/release107/Homo_sapiens.GRCh38.107.protein_coding.chr.gtf.gz'
-            gtf_db_path = path.dirname(__file__) + '/data/Ensembl/release107/Homo_sapiens.GRCh38.107.protein_coding.chr.gtf.db'
+            if verbose:log.write(" -Assigning Gene name using built-in Ensembl Release hg38")
+            gtf_path = check_and_download("ensembl_hg38_gtf_protein_coding")
+            gtf_db_path = gtf_path[:-2]+"db"
             data = Genome(
                 reference_name='GRCh38',
                 annotation_name='hg38_gene',
@@ -235,9 +240,9 @@ def annogene(
     
     if source == "refseq":
         if build=="19":
-            if verbose:log.write(" -Assigning Gene name using built-in NCBI refseq latest GRCh37")
-            gtf_path = path.dirname(__file__) + '/data/RefSeq/GRCh37/GRCh37_latest_genomic.protein_coding.gtf.gz'
-            gtf_db_path = path.dirname(__file__) + '/data/RefSeq/GRCh37/GRCh37_latest_genomic.protein_coding.gtf.db'
+            if verbose:log.write(" -Assigning Gene name using NCBI refseq latest GRCh37")
+            gtf_path = check_and_download("refseq_hg19_gtf_protein_coding")
+            gtf_db_path = gtf_path[:-2]+"db"
             data = Genome(
                 reference_name='GRCh37',
                 annotation_name='hg19_gene',
@@ -249,8 +254,8 @@ def annogene(
                 index=output.index).values
         elif build=="38":
             if verbose:log.write(" -Assigning Gene name using built-in NCBI refseq latest GRCh38")
-            gtf_path = path.dirname(__file__) + '/data/RefSeq/GRCh38/GRCh38_latest_genomic.protein_coding.gtf.gz'
-            gtf_db_path = path.dirname(__file__) + '/data/RefSeq/GRCh38/GRCh38_latest_genomic.protein_coding.gtf.db'
+            gtf_path = check_and_download("refseq_hg38_gtf_protein_coding")
+            gtf_db_path = gtf_path[:-2]+"db"
             data = Genome(
                 reference_name='GRCh38',
                 annotation_name='hg38_gene',
@@ -264,12 +269,13 @@ def annogene(
     return output
 
 def getnovel(insumstats,
-           known,
            id,
            chrom,
            pos,
            p,
-           only_novel=True,
+           known=None,
+           efo=None,
+           only_novel=False,
            windowsizekb_for_novel=1000,
            windowsizekb=500,
            sig_level=5e-8,
@@ -278,6 +284,7 @@ def getnovel(insumstats,
            anno=False,
            build="19",
            source="ensembl",
+           gwascatalog_source="NCBI",
            verbose=True):
     
     allsig = getsig(insumstats=insumstats,
@@ -286,12 +293,19 @@ def getnovel(insumstats,
     
     allsig["TCHR+POS"]=allsig[chrom]*1000000000 + allsig[pos]
     
-    if type(known) is pd.DataFrame:
+    if known is None and efo is not None:
+        known = gwascatalog_trait(efo,source=gwascatalog_source,sig_level=sig_level,verbose=verbose,log=log)
+        knownsig = known.data.copy()
+        knownsig["CHR"] = knownsig["CHR"].astype("Int64")
+        knownsig["POS"] = knownsig["POS"].astype("Int64")
+    elif type(known) is pd.DataFrame:
         knownsig = known.copy()
         knownsig["CHR"] = knownsig["CHR"].astype("Int64")
         knownsig["POS"] = knownsig["POS"].astype("Int64")
     elif type(known) is str:
         knownsig = pd.read_csv(known,sep="\s+",dtype={"CHR":"Int64","POS":"Int64"})
+    else:
+        raise ValueError("Please input a dataframe of known loci or efo code")
     
     if "SNPID" in knownsig.columns:
         knownids=knownsig["SNPID"].values
@@ -323,7 +337,8 @@ def getnovel(insumstats,
     if verbose: log.write(" -Identified ",len(allsig)-sum(allsig["NOVEL"])," known vairants in current sumstats...")
     if verbose: log.write(" -Identified ",sum(allsig["NOVEL"])," novel vairants in current sumstats...")
     if verbose: log.write("Finished checking known or novel successfully!")
+    gc.collect()
     if only_novel is True:
         return allsig.loc[allsig["NOVEL"],:]
     else:
-        return allsig
\ No newline at end of file
+        return allsig
