@@ -5,6 +5,7 @@ import seaborn as sns
 import numpy as np
 import scipy as sp
 import gwaslab as gl
+from shutil import which
 from gwaslab.Log import Log
 from gwaslab.getsig import getsig
 from gwaslab.getsig import annogene
@@ -19,6 +20,7 @@ from allel import rogers_huff_r_between
 import matplotlib as mpl
 from scipy import stats
 from mpl_toolkits.axes_grid1.inset_locator import inset_axes
+import gc as garbage_collect
 
 # 20220310 ######################################################################################################
 
@@ -51,6 +53,7 @@ def mqqplot(insumstats,
           region_anno_bbox_args={},
           taf=[4,0,0.95,1,1],
           # track_n, track_n_offset,font_ratio,exon_ratio,text_offset
+          tabix=None,
           mqqratio=3,
           bwindowsizekb = 100,
           large_number = 10000000000,
@@ -85,7 +88,7 @@ def mqqplot(insumstats,
           title =None,
           mtitle=None,
           qtitle=None,
-          figargs= {"figsize":(15,5),"dpi":100},
+          figargs= {"figsize":(15,5)},
           fontsize = 10,
           colors=["#597FBD","#74BAD3"],
           marker_size=(5,25),
@@ -94,15 +97,15 @@ def mqqplot(insumstats,
           repel_force=0.03,
           build="19",
           title_pad=1.08, 
-          dpi=100,
+          dpi=200,
           save=None,
-          saveargs={"dpi":400,"facecolor":"white"},
+          saveargs={"dpi":300,"facecolor":"white"},
           _invert=False,
           log=Log()
           ):
-    
+
 # log.writeing meta info #######################################################################################
-    
+
     if verbose: log.write("Start to plot manhattan/qq plot with the following basic settings:")
     if verbose: log.write(" -Genome-wide significance level is set to "+str(sig_level)+" ...")
     if verbose: log.write(" -Raw input contains "+str(len(insumstats))+" variants...")
@@ -116,9 +119,9 @@ def mqqplot(insumstats,
         if verbose: log.write(" -Variants to pinpoint : "+",".join(pinpoint))  
     if region is not None:
         if verbose: log.write(" -Region to plot : chr"+str(region[0])+":"+str(region[1])+"-"+str(region[2])+".")  
-    if dpi!=100:
+    if "dpi" not in figargs.keys():
         figargs["dpi"] = dpi
-    
+
 
 # Plotting mode selection : layout ####################################################################
     # ax1 : manhattanplot : 
@@ -156,10 +159,10 @@ def mqqplot(insumstats,
         marker_size= (marker_size[1],marker_size[1])
     else:
         raise ValueError("Please select one from the 5 modes: mqq/qqm/m/qq/r/b")
-        
-        
+
+
 # Read sumstats #################################################################################################
-    
+
     usecols=[]
     #  P ###############################################################################
     if p in insumstats.columns:
@@ -173,6 +176,7 @@ def mqqplot(insumstats,
         raise ValueError("Please make sure "+p+" column is in input sumstats.")
     
     # CHR and POS ########################################################################
+    # chrom and pos exists && (m || r mode)
     if (chrom is not None) and (pos is not None) and (("m" in mode) or ("r" in mode)):
         # when manhattan plot, chrom and pos is needed.
         if chrom in insumstats.columns:
@@ -216,7 +220,7 @@ def mqqplot(insumstats,
     
     sumstats = insumstats.loc[:,usecols].copy()
 
-#####################################################################################################################   
+
     #Standardize
     ## Annotation
     if (anno == "GENENAME"):
@@ -225,14 +229,16 @@ def mqqplot(insumstats,
         sumstats["Annotation"]=sumstats.loc[:,anno].astype("string")   
       
     ## P value
-    if "m" in mode and "b" not in mode:   
+    ## m, qq, r
+    if "b" not in mode:   
         if scaled is True:
             sumstats["raw_P"] = pd.to_numeric(sumstats[mlog10p], errors='coerce')
         else:
             sumstats["raw_P"] = sumstats[p].astype("float64")
     
     ## CHR & POS
-    if "m" in mode: 
+    ## m, qq, b
+    if "m" in mode or "r" in mode or "b" in mode: 
         # CHR X,Y,MT conversion ############################
         if sumstats[chrom].dtype =="str":
             sumstats[chrom] = sumstats[chrom].map(get_chr_to_number(),na_action="ignore")
@@ -240,6 +246,8 @@ def mqqplot(insumstats,
         sumstats[chrom] = np.floor(pd.to_numeric(sumstats[chrom], errors='coerce')).astype('Int64')
         ## POS
         sumstats[pos] = np.floor(pd.to_numeric(sumstats[pos], errors='coerce')).astype('Int64')
+    
+    ## r
     if region is not None:
         region_chr = region[0]
         region_start = region[1]
@@ -250,6 +258,7 @@ def mqqplot(insumstats,
         in_region_snp = (sumstats[chrom]==region_chr) &(sumstats[pos]<region_end) &(sumstats[pos]>region_start)
         if verbose:log.write(" -Extract SNPs in specified regions: "+str(sum(in_region_snp)))
         sumstats = sumstats.loc[in_region_snp,:]
+    
     ## EAF
     if stratified is True: 
         sumstats["MAF"] = pd.to_numeric(sumstats[eaf], errors='coerce')
@@ -259,11 +268,11 @@ def mqqplot(insumstats,
         sumstats["HUE"] = sumstats[chrom].astype("string")
     
     if verbose: log.write("Finished loading specified columns from the sumstats.")
-        
-        
+
+
 #sanity check############################################################################################################
     if verbose: log.write("Start conversion and sanity check:")
-    if "m" in mode: 
+    if ("m" in mode or "r" in mode): 
         pre_number=len(sumstats)
         #sanity check : drop variants with na values in chr and pos df
         sumstats = sumstats.dropna(subset=[chrom,pos])
@@ -277,7 +286,7 @@ def mqqplot(insumstats,
         if verbose:log.write(" -Removed "+ str(pre_number-after_number) +" variants with nan in EAF column ...")
             
         ## Highlight
-    if len(highlight)>0 and ("m" in mode):
+    if len(highlight)>0 and ("m" in mode or "r" in mode):
         to_highlight = sumstats.loc[sumstats[snpid].isin(highlight),:]
         #assign colors: 0 is hightlight color
         for i,row in to_highlight.iterrows():
@@ -309,12 +318,14 @@ def mqqplot(insumstats,
      
     #############################
 # P value conversion #####################################################################################################  
+    ## m,qq,r -> dropna
     if "b" not in mode:
         pre_number=len(sumstats)
         sumstats = sumstats.dropna(subset=["raw_P"])
         after_number=len(sumstats)
         if verbose:log.write(" -Removed "+ str(pre_number-after_number) +" variants with nan in P column ...")
     
+    ## b: value to plot is density
     if "b" in mode:
         sumstats["scaled_P"] = sumstats["DENSITY"].copy()
         sumstats["raw_P"] = -np.log10(sumstats["DENSITY"].copy()+2)
@@ -340,7 +351,7 @@ def mqqplot(insumstats,
     
     # filter out variants with -log10p < skip
     sumstats = sumstats.loc[sumstats["scaled_P"]>=skip,:]
-    
+    garbage_collect.collect()
     
     # shrink variants above cut line #########################################################################################
     maxy = sumstats["scaled_P"].max()
@@ -370,17 +381,19 @@ def mqqplot(insumstats,
             maxy = (maxticker-cut)/cutfactor + cut
     if verbose: log.write("Finished data conversion and sanity check.")
 
-        
+
 # Manhattan plot ##########################################################################################################
 ## regional plot ->rsq
      #calculate rsq]
     if vcf_path is not None:
+        if tabix is None:
+            tabix = which("tabix")
         sumstats = process_vcf(sumstats=sumstats, vcf_path=vcf_path,region=region, 
-                               log=log ,pos=pos,region_ld_threshold=region_ld_threshold,verbose=verbose,vcf_chr_dict=vcf_chr_dict)
+                               log=log ,pos=pos,region_ld_threshold=region_ld_threshold,verbose=verbose,vcf_chr_dict=vcf_chr_dict,tabix=tabix)
+
+# # Create index for plotting ###################################################
+
 
-## Create index for plotting ###################################################
-        
-    
     #sort & add id
     if ("m" in mode) or ("r" in mode): 
         sumstats = sumstats.sort_values([chrom,pos])
@@ -644,7 +657,7 @@ def mqqplot(insumstats,
             ax1.set_ylim(bottom = skip)
 
 
-    
+
 # Get top variants for annotation #######################################################
         if (anno and anno!=True) or (len(anno_set)>0):
             if len(anno_set)>0:
@@ -690,7 +703,7 @@ def mqqplot(insumstats,
             #    if verbose:log.write(" -Assigning Gene name using Ensembl Release",77)
             #to_annotate = to_annotate.copy()
             #to_annotate.loc[:,["LOCATION","Annotation"]] = pd.DataFrame(to_annotate.apply(lambda x:closest_gene(x,data=data), axis=1).tolist(), index=to_annotate.index).values
-        
+
 # Add Annotation to manhattan plot #######################################################
            ## final
         
@@ -840,9 +853,9 @@ def mqqplot(insumstats,
             
         else:
             if verbose: log.write(" -Skip annotating")
-        
 
-     
+
+
 # Creating Manhatann plot Finished #####################################################################
 
 # QQ plot #########################################################################################################
@@ -925,18 +938,20 @@ def mqqplot(insumstats,
         fig.suptitle(title ,x=0.5, y=1.05)
     else:
         fig.suptitle(title ,x=0.5,y=1)
-
+    
+    garbage_collect.collect()
 # Return matplotlib figure object #######################################################################################
     return fig, log
 
 
+# +
 #Helpers
-##############################################################################################################################
-def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_threshold, vcf_chr_dict):
+
+def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_threshold, vcf_chr_dict,tabix):
     if verbose: log.write("Start to load reference genotype...")
     if verbose: log.write(" -reference vcf path : "+ vcf_path)
     # load genotype data of the targeted region
-    ref_genotype = read_vcf(vcf_path,region=vcf_chr_dict[region[0]]+":"+str(region[1])+"-"+str(region[2]),tabix=None)
+    ref_genotype = read_vcf(vcf_path,region=vcf_chr_dict[region[0]]+":"+str(region[1])+"-"+str(region[2]),tabix=tabix)
     
     if verbose: log.write(" -Retrieving index...")
     # match sumstats pos and ref pos: 
@@ -977,7 +992,10 @@ def process_vcf(sumstats, vcf_path, region, log, verbose, pos , region_ld_thresh
     if verbose: log.write("Finished loading reference genotype successfully!")
     return sumstats
 
-##############################################################################################################################
+
+# -
+
+""
 def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding):
     #loading
     if gtf_path =="defualt" or gtf_path =="ensembl":
@@ -1047,7 +1065,7 @@ def process_gtf(gtf_path,region,region_flank_factor,build,region_protein_coding)
     return uniq_gene_region, exons
 
 
-##############################################################################################################################
+""
 def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50):
         gene = data.gene_names_at_locus(contig=x[chrom], position=x[pos])
         if len(gene)==0:
@@ -1073,4 +1091,4 @@ def closest_gene(x,data,chrom="CHR",pos="POS",maxiter=20000,step=50):
                     i+=1
             return distance,"intergenic"
         else:
-            return 0,",".join(gene)
\ No newline at end of file
+            return 0,",".join(gene)
